#ifndef POKEPLATINUM_UNK_020508D4_H
#define POKEPLATINUM_UNK_020508D4_H

#include "struct_decls/struct_0203CDB0_decl.h"
#include "struct_decls/struct_020508D4_decl.h"
#include "functypes/funcptr_02050904.h"
#include "struct_defs/struct_0208BE5C.h"

UnkStruct_020508D4 * sub_02050904(UnkStruct_0203CDB0 * param0, UnkFuncPtr_02050904 param1, void * param2);
void sub_02050924(UnkStruct_020508D4 * param0, UnkFuncPtr_02050904 param1, void * param2);
UnkStruct_020508D4 * sub_02050944(UnkStruct_020508D4 * param0, UnkFuncPtr_02050904 param1, void * param2);
BOOL sub_02050958(UnkStruct_0203CDB0 * param0);
BOOL sub_020509A4(UnkStruct_0203CDB0 * param0);
BOOL sub_020509B4(UnkStruct_0203CDB0 * param0);
void sub_020509D4(UnkStruct_0203CDB0 * param0);
BOOL sub_020509DC(UnkStruct_0203CDB0 * param0);
void sub_02050A38(UnkStruct_020508D4 * param0, const UnkStruct_0208BE5C * param1, void * param2);
UnkStruct_0203CDB0 * sub_02050A60(UnkStruct_020508D4 * param0);
void * sub_02050A64(UnkStruct_020508D4 * param0);
int * sub_02050A68(UnkStruct_020508D4 * param0);
u32 sub_02050A6C(UnkStruct_020508D4 * param0);

#endif // POKEPLATINUM_UNK_020508D4_H
